require_relative 'codebreaker_vukolov_edition/version'
require_relative 'game'
require_relative 'messages'
require_relative 'saves'

module CodebreakerVukolovEdition
  class Controller
    AVAILIABLE_VALUES = %w[hint yes no]
    attr_accessor :game, :services

    def initialize
      @game = Game.new
      @messages = Messages.new
    end

    def play
      return total_result(:lose) if @game.turns == Game::MAX_TURNS
      @messages.before_round_info(@game.turns)
      show_hint
      guess = provided_guess
      result = @game.round_result(guess)
      round_info(result)
      @game.add_new_turn
      play
    end

    def round_info(result)
      return total_result(:won) if result == :won
      @messages.show_messages(:round_result, result)
    end

    def show_hint
      return @messages.print_hint(@game.hint) if @game.hint[:used]
      @messages.hint_info
    end

    def next_step(decision, *option)
      if decision == 'yes'
        yield
      elsif decision == 'no' && option.include?('save_result')
        @messages.show_messages(:game_unsaved)
      else
        abort
      end
    end

    def total_result(state)
      @messages.total_game_info(state, @game.turns, @game.secret_code)
      next_step(make_a_decision(:save_result), 'save_result') do
        username = get_action(:name)
        save = Saves.new(username, state, @game)
        save.write_to_file
        @messages.show_messages(:result_saved, CodebreakerVukolovEdition::Saves::FILENAME)
      end
      next_step(make_a_decision(:new_game)) do
        game = CodebreakerVukolovEdition::Controller.new
        game.play
      end
    end

    def make_a_decision(option)
      decision = validate(get_action(option))
      return decision unless decision.nil?
      @messages.show_messages(:instruction)
      make_a_decision(option)
    end

    def get_action(*question)
      @messages.show_messages(question.first) if question
      value = gets.chomp
      value.empty? ? get_action(*question) : value
    end

    def provided_guess
      input = validate(get_action(:input))
      if input.nil?
        @messages.show_messages(:input_warning)
        provided_guess
      elsif input == 'hint'
        @game.use_hint
        play
      else
        input
      end
    end

    private

    def validate(input)
      input if (input.to_i.positive? && input.length == 4) || AVAILIABLE_VALUES.include?(input)
    end
  end
end
