require_relative 'codebreaker_vukolov_edition'

module CodebreakerVukolovEdition
  class Game
    SECRET_CODE_VALID_VALUES = (1..6)
    SECRET_CODE_LENGTH = 4
    MAX_TURNS = 15
    attr_accessor :turns, :secret_code, :hint

    def initialize
      @secret_code = new_code
      @turns = 0
      @hint = { used: false, value: @secret_code.sample }
    end

    def add_new_turn
      @turns += 1
    end

    def use_hint
      @hint[:used] = true
    end

    def round_result(guess)
      return :won if @secret_code.join('') == guess
      checker(guess)
    end

    def checker(guess)
      guess_arr = guess.split('').map(&:to_i)
      temp = @secret_code.zip(guess_arr)
      get_plusses(temp) + get_minuses(temp)
    end

    private

    def new_code
      Array.new(SECRET_CODE_LENGTH).map { rand(SECRET_CODE_VALID_VALUES) }
    end

    def count_plusses(temp)
      temp.select { |el| el.uniq.size == 1 }
    end

    def get_plusses(temp)
      '+' * count_plusses(temp).size
    end

    def get_minuses(temp)
      minuses = ''
      code = (temp - count_plusses(temp)).transpose.first
      user_guess = (temp - count_plusses(temp)).transpose.last
      code&.each do |code_value|
        next unless user_guess.include?(code_value)
        user_guess.delete_at(user_guess.index(code_value) || user_guess.length) && minuses << '-'
      end
      minuses
    end
  end
end
